;;;; -*- Mode: Lisp -*-
;;;; $Id: usocket-cmucl.lisp 449 2008-09-04 13:16:39Z binghe $

(in-package :usocket)

(defun socket-connect/udp (host port &key (stream nil) (element-type '(unsigned-byte 8))
			                  local-host local-port)
  (if (and host port)
      (let ((socket (ext:connect-to-inet-socket (host-to-hbo host) port :datagram
						:local-host (if local-host
								(host-to-hbo local-host))
						:local-port local-port)))
	(if stream
	    (make-stream-datagram-socket socket
					 (system:make-fd-stream socket
								:input t :output t
								:buffering :full
								:element-type element-type))
	    (make-datagram-socket socket)))
      (make-datagram-socket (let ((socket (ext:create-inet-socket :datagram)))
			      (when (and local-host local-port)
				(ext:bind-inet-socket socket local-host local-port))
			      socket))))

(defmethod socket-send ((socket datagram-usocket) buffer length &key address port)
  (let ((s (socket socket)))
    (ext:inet-sendto s buffer length (if address (host-to-hbo address)) port)))

(defmethod socket-receive ((socket datagram-usocket) buffer length)
  (let ((s (socket socket)))
    (let ((real-buffer (or buffer
                           (make-array length :element-type '(unsigned-byte 8))))
          (real-length (or length
                           (length buffer))))
      (multiple-value-bind (nbytes remote-host remote-port)
	  (ext:inet-recvfrom s real-buffer real-length)
	(if (plusp nbytes)
	    (values real-buffer nbytes remote-host remote-port)
	    (error "CMUCL: socket-receive failed"))))))
